# -*- coding: UTF-8 -*-

from django.db import models
from django.db.models.signals import post_delete
from django.dispatch import receiver
from django.utils import timezone

from django_simple_files.utils import get_upload_to


class Files(models.Model):
    """
    文件DB存储数据
    """

    class Meta:
        verbose_name_plural = '文件存档'
        verbose_name = '文件存档'
        ordering = ['-file_id']

    file_name = models.CharField('文件名', max_length=255)
    file_id = models.AutoField('ID', primary_key=True)
    file_uri = models.FileField('文件', max_length=255, upload_to=get_upload_to(),)
    file_size = models.BigIntegerField('文件大小', default=0)
    file_hash = models.CharField('文件hash', max_length=40)
    time_created = models.DateTimeField('上传时间', default=timezone.now)
    uploader_uid = models.IntegerField('上传者UID', default=0)
    enabled = models.BooleanField('使用中', default=True)

    def __unicode__(self):
        return self.file_uri.name

    def __str__(self):
        return self.file_uri.name


@receiver(post_delete, sender=Files)
def delete_file_handle(sender, **kwargs):
    """
    绑定删除事件
    :param sender:
    :param kwargs:
    :return:
    """
    _instance = kwargs.get('instance')
    _instance.file_uri.delete(save=False)
    # FilesRelated.objects.filter(file=Files).delete()



class FilesRelated(models.Model):
    """
    文件关系存储数据
    """

    class Meta:
        verbose_name_plural = '文件关系存储'
        verbose_name = '文件关系存储'
        ordering = ['-related_id']

    related_id = models.AutoField('ID', primary_key=True)
    file = models.ForeignKey(Files)
    file_uri = models.CharField('文件', max_length=255)
    related_object_id = models.CharField('对象PK', max_length=191, db_index=True)
    related_object = models.CharField('对象名', max_length=100, db_index=True)
    time_created = models.DateTimeField('上传时间', default=timezone.now)

    def __unicode__(self):
        return self.file_uri

    def __str__(self):
        return self.file_uri





# -*- coding: UTF-8 -*-
from django.contrib import admin

from django_simple_files import api
from . import models


@admin.register(models.Files)
class FilesAdmin(admin.ModelAdmin):
    list_display = ['file_id','file_name', 'file_uri', 'file_size', 'file_hash', 'enabled',
                    'time_created', 'uploader_uid']
    readonly_fields = ['file_size', 'file_hash', 'uploader_uid']

    def save_model(self, request, obj, form, change):
        file = request.FILES.get('file_uri')
        api.save(file.name, file.read(), request.user.id, obj.file_id if obj.file_id else 0)




# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-07-24 15:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_simple_files', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='files',
            name='enabled',
            field=models.BooleanField(default=True, verbose_name='使用中'),
        ),
    ]



# -*- coding: UTF-8 -*-

import base64
import binascii
import hashlib
import logging
import os
import uuid

from django_simple_files import settings
from django_simple_files.settings import SIMPLE_FILES

logger = logging.getLogger('django')


def crc32(file_blob):
    """
    从进制获取文件crc32信息
    :param file_blob:
    :return:
    """
    crc_code = binascii.crc32(file_blob)
    if crc_code > 0:
        crc_int = crc_code
    else:
        crc_int = ~ crc_code ^ 0xffffffff
    return '%x' % crc_int


def crc32_from_path(file_path):
    """
    从路径文件crc32信息
    :param file_path:
    :return:
    """
    with open(file_path, 'rb') as f:
        return crc32(f.read())


def blob_from_h5base64(blob_from_h5base64_data):
    """
    从前端预览上传的Base64编码获取data数据
    :param blob_from_h5base64_data:
    :return:
    """
    return blob_from_base64(blob_from_h5base64_data.split(',')[-1])


def blob_from_base64(base64_data):
    """
    从Base64完整data中获取二进制data数据
    :param base64_data:
    :return:
    """
    return base64.b64decode(base64_data)


def md5(file_blob):
    """
    从二进制对象获取md5信息
    :param file_blob:
    :return:
    """
    return hashlib.md5(file_blob).hexdigest()


def md5_from_path(file_path):
    """
    从文件获取md5信息
    :param file_path:
    :return:
    """
    with open(file_path, "rb") as f:
        return md5(f.read())


def sha1(file_blob):
    """
    从二进制对象获取md5信息
    :param file_blob:
    :return:
    """
    return hashlib.sha1(file_blob).hexdigest()


def sha1_from_path(file_path):
    """
    从文件获取md5信息
    :param file_path:
    :return:
    """
    with open(file_path, "rb") as f:
        return sha1(f.read())


def hash_uuid():
    return str(uuid.uuid1()).replace('-', '')


def get_object_name(obj):
    return '%s.%s' % (obj.__module__, obj.__class__.__name__)


def get_file_hash(content):
    hash_fun = get_file_fun_name()
    if hash_fun == "CRC32":
        return crc32(content)
    elif hash_fun == "SHA1":
        return sha1(content)
    else:
        return md5(content)


def get_uuid_name(name):
    hash_name = hash_uuid()
    file_ext = os.path.splitext(name)[-1]
    return hash_name + file_ext


# settings

def get_setting_value(name):
    simple_files_name = 'SIMPLE_FILES'
    if hasattr(settings, simple_files_name):
        simple_files_dict = getattr(settings, simple_files_name)
        if isinstance(simple_files_dict, dict):
            return simple_files_dict.get(name) or SIMPLE_FILES.get(name)


def get_file_fun_name():
    file_hash_fun_key = 'FILE_HASH_FUN'
    file_hash_fun = get_setting_value(file_hash_fun_key)
    return file_hash_fun if file_hash_fun and file_hash_fun == 'CRC32' else SIMPLE_FILES.get(file_hash_fun_key)


def get_upload_to():
    file_upload_to_key = 'FILE_UPLOAD_TO'
    upload_to = get_setting_value(file_upload_to_key)
    return upload_to if upload_to else SIMPLE_FILES.get(file_upload_to_key)


def get_file_auto_hashname():
    file_auto_hashname_key = 'FILE_AUTO_HASHNAME'
    file_auto_hashname = get_setting_value(file_auto_hashname_key)
    return file_auto_hashname if file_auto_hashname else SIMPLE_FILES.get(file_auto_hashname_key)



# -*- coding: utf-8 -*-
# Created by lvjiyong on 16/8/20
import logging

from django.core.files.base import ContentFile
from django.core.files.storage import default_storage
from django_simple_files.models import Files

logger = logging.getLogger('django')


def save(name, content, storage_instance=None):
    """
    使用django storage 保存文件
    :param storage_instance:
    :param name:
    :param content:
    :return:
    """

    if isinstance(storage_instance, Files):
        storage_instance.file_uri.save(name, ContentFile(content))
        return storage_instance.file_uri.name
    elif not storage_instance:
        logger.debug(storage_instance)
        storage_instance = default_storage
    return storage_instance.save(name, ContentFile(content))


def exists(name):
    return default_storage.exists(name)


def delete(name):
    return default_storage.delete(name)
